/***************************************************************************
 *   Copyright (C) 2015-2016 Helge Kortendieck  <alan@kasmithee.de>        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef _LAME_H_
#define _LAME_H_


#include <QByteArray>
#include <QFileInfo>
#include <QObject>
#include <QProcess>
#include <QSettings>
#include <QString>
#include <QStringList>
#include <QTime>

#include "structs.h"

#include <iostream>
using namespace std;

class lame : public QObject
{
	Q_OBJECT

public:
	lame(QObject *parent = 0);
		~lame();

public slots:

protected:
    /*$PROTECTED_FUNCTIONS$*/

private:
	bool bD;
	bool bD2;
	bool bD3;
	bool bDelSpaces;
	bool bShorten;
	bool bSkip;
	bool bWavDoNotReC;

	int iD;
	int iLameCmd;			// 0=undefined, 1=decoding, 2=encoding
	int iLoop;				// loop counter
	int iNumFilesToProc;	// # files to process

	QByteArray aOutput;		// qprocess output, stdout or stderr

	QFileInfo fiL;
	QFileInfo fiL2;
	QFileInfo fiL3;

	QProcess *qpLame;		// qprocess calling lame

	QString sD;
	QString sD2;

	QString sDirIn;			// directory with files to process
	QString sDirOut;		// directory for output files, full path and name incl. '/' at end
	QString sDirSrc;		// source dir with ending '/'
	QString sFileOut;		// full path and name of output file
	QString sFileSrc;		// full path and name of src file
//	QString sWavName;		// name of wav, excl. path
	QString sTmpDir;

	QString sLameQuality;

	QString sIn;					// incomming, here process output
	QString sLameCmd;				// lame command, decode/encode

	QStringList slFilesToProc;		// files to process, e. g. mp3s to decode
	QStringList slArgs;				// qprocess arguments
	QStringList slReturn;			// list with files that failed or general errors (no files to process, directory permissino problems)

	FilePairs fpLD;
	QList<FilePairs> liLFilePairs;	// list struct with file name pairs successfully decoded from mp3 to wav or encoded from wav to mp3

private slots:
	void lameDo(const QString&, const QStringList& slD, const QString& sD, int iD, bool bD, bool bD2, bool bD3);
								// src dir, file list with mp3s/wavs to process (name), output dir, command 1=decode/2=encode, randomize, del spaces, shorten to 8

	void procLoop();			// run lame as often as it takes to decode all mp3s to wavs or encode wacs to mp3s
		void procRun();			// start qprocess to either decode or encode
		void procStdOut();		// read qprocess stdout
		void procStdErr();		// read qprocess stderr
		void procEnd(int iD, QProcess::ExitStatus iD2);	// qprocess ended
		void procKill();		// kill qprocess, asked to stop
	void procLoopEnded();		// all files processed, return

	void settingsRead();		// read lame encoding quality settings

	signals:
		void sigLameDecodeEnded(const QStringList&, const QList<FilePairs>&);	// lame qprocess ended, mp3 to wav
		void sigLameFileProc(const QString&);									// update Gui with currently processed file
		void sigLameEncodeEnded(const QStringList&, const QList<FilePairs>&);	// lame qprocess ended, wav to mp3
};

#endif		// _LAME_H_
