/****************************************************************************
 **
 **  Copyright (C) 2015-2016 Helge Kortendieck, alan@kasmithee.de
 **
 **  Main program, handle GUI, call QProcesses for lame and wavegain.
 **  Control tag re-creation by taglib.
 **
 **  This file may be used under the terms of the GNU General Public
 **  License version 2.0 or later as published by the Free Software Foundation
 **
 **  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 **  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/

#include "ntg.h"

Qntg::Qntg(QMainWindow *parent)
	: QMainWindow(parent)
{
	bQuit = false;
	bTaglib = false;
	iRunLevel = 0;
	iRunState = 0;			// unable to run
	iWShown = 0;
	iWToShow = 0;
	iDirCreatedWav = 2;		// did not create the wav directory - no clean up
	iDirCreatedOut = 2;		// did not create the output directory - no clean up - currently no GUI option to select cleaning up normalized mp3 dir !

	actionsCreate();

	ui.setupUi(this);

	menuBarCreate();
	toolBar = addToolBar(tr("Show toolbar"));
	toolBarCreate();
	statusBarCreate();

	connect(ui.cB_WavCreateSubDir, SIGNAL(toggled(bool)), this, SLOT(wavCreateSubDirToggled()));
	connect(ui.cB_OutCreateSubDir, SIGNAL(toggled(bool)), this, SLOT(outCreateSubDirToggled()));

	lm = new lame();
	connect(this, SIGNAL(sigLameDo(const QString&, const QStringList&, const QString&, int, bool, bool, bool)), lm, SLOT(lameDo(const QString&, const QStringList&, const QString&, int, bool, bool, bool)));
	connect(lm, SIGNAL(sigLameFileProc(const QString&)), this, SLOT(runGuiUpdFileProc(const QString&)));
	connect(lm, SIGNAL(sigLameDecodeEnded(const QStringList&, const QList<FilePairs>&)), this, SLOT(run3LameDecodeToWavReturn(const QStringList&, const QList<FilePairs>&)));
	connect(lm, SIGNAL(sigLameEncodeEnded(const QStringList&, const QList<FilePairs>&)), this, SLOT(run5EncodeWavToMp3Return(const QStringList&, const QList<FilePairs>&)));

	tl = new tags();
	connect(this, SIGNAL(sigTaglibReAdd(const QList<FilePairs>&)), tl, SLOT(tagsReAdd(const QList<FilePairs>&)));
	connect(tl, SIGNAL(sigTagsFileProc(const QString&)), this, SLOT(runGuiUpdFileProc(const QString&)));
	connect(tl, SIGNAL(sigTagsReAddEnded(const QStringList&)), this, SLOT(run7ReAddTagsReturn(const QStringList&)));

	wg = new wavegain();
	connect(this, SIGNAL(sigWavegainStart(const QString&)), wg, SLOT(procPrep(const QString&)));
	connect(wg, SIGNAL(sigNormFileProc(const QString&)), this, SLOT(runGuiUpdFileProc(const QString&)));
	connect(wg, SIGNAL(sigNormEnded(bool)), this, SLOT(run4WavegainNormalizeReturn(bool)));

/*	connect(ui.lE_Path, SIGNAL(editingFinished()), this, SLOT(destDirEditFin()));
	connect(ui.pB_Path, SIGNAL(clicked()), this, SLOT(destDirSetClicked()));
	connect(ui.lE_File, SIGNAL(editingFinished()), this, SLOT(outNameLeEditFin()));
	connect(ui.pB_File, SIGNAL(clicked()), this, SLOT(outNameDialogClicked()));
*/
	ui.pB_Wav->setIcon(QIcon("/usr/share/normalizetogo/icons/scalable/open.svg"));
	ui.pB_Out->setIcon(QIcon("/usr/share/normalizetogo/icons/scalable/open.svg"));

	settingsRead();

	initDelay();								// short timer to allow GUI to come up before proceeding...
}

Qntg::~Qntg()
{
}

// ****************************************
// **  settings read / write			 **
// ****************************************
void Qntg::settingsRead()
{
	QSettings settings("normalizetogo", "normalizetogo");

	settings.beginGroup("mainwindow");
		QPoint pos = settings.value("pos", QPoint(0, 0)).toPoint();
		QSize size = settings.value("size", QSize(740, 540)).toSize();
	settings.endGroup();

	settings.beginGroup("src");
		ui.tV_FileSys->header()->restoreState(settings.value("tvheaderstate", QByteArray()).toByteArray());
		iTvCw0 = settings.value("tV_FileSys-widthcol0", 50).toInt();		// tree view: width column 1
		iTvCw1 = settings.value("tV_FileSys-widthcol1", 50).toInt();		// tree view: width column 2
		iTvCw2 = settings.value("tV_FileSys-widthcol2", 50).toInt();		// tree view: width column 3
		iTvCw3 = settings.value("tV_FileSys-widthcol3", 50).toInt();		// tree view: width column 4
		sSrcDir = settings.value("srcdir", "/").toString();
	settings.endGroup();

	settings.beginGroup("wav");
		ui.lE_Wav->setText(settings.value("wavdir", "/tmp").toString());
		ui.cB_WavCreateSubDir->setChecked(settings.value("wavcreatesubdir", "false").toBool());
		ui.cB_WavDelSubDir->setChecked(settings.value("wavdelsubdir", "false").toBool());
		ui.cB_WavNotReCreate->setChecked(settings.value("wavnotrecreate", "true").toBool());
		ui.cB_WavDelFiles->setChecked(settings.value("wavdelfiles", "false").toBool());
	settings.endGroup();

	settings.beginGroup("out");
		ui.lE_Out->setText(settings.value("outdir", "/tmp").toString());
		ui.cB_OutCreateSubDir->setChecked(settings.value("outcreatesubdir", "false").toBool());
		ui.cB_OutDelSpaces->setChecked(settings.value("outdelspaces", "false").toBool());
		ui.cB_OutShorten->setChecked(settings.value("outshortento8", "false").toBool());
		ui.cB_OutRandomize->setChecked(settings.value("outrandomize", "false").toBool());
		ui.cB_OutTagsReAdd->setChecked(settings.value("outtagsreadd", "true").toBool());
	settings.endGroup();

	resize(size);
	move(pos);
}

void Qntg::settingsWrite()
{
	QSettings settings("normalizetogo", "normalizetogo");

	settings.beginGroup("mainwindow");
		settings.setValue("pos", pos());
		settings.setValue("size", size());
	settings.endGroup();

	settings.beginGroup("src");
		settings.setValue("tvheaderstate", ui.tV_FileSys->header()->saveState());
		settings.setValue("tV_FileSys-widthcol0", ui.tV_FileSys->columnWidth(0));   // src: width column 1
		settings.setValue("tV_FileSys-widthcol1", ui.tV_FileSys->columnWidth(1));   // src: width column 2
		settings.setValue("tV_FileSys-widthcol2", ui.tV_FileSys->columnWidth(2));   // src: width column 3
		settings.setValue("tV_FileSys-widthcol3", ui.tV_FileSys->columnWidth(3));   // src: width column 4
		settings.setValue("srcdir", sSrcDir);
	settings.endGroup();

	settings.beginGroup("wav");
		settings.setValue("wavdir", ui.lE_Wav->text());
		settings.setValue("wavcreatesubdir", ui.cB_WavCreateSubDir->isChecked());
		settings.setValue("wavdelsubdir", ui.cB_WavDelSubDir->isChecked());
		settings.setValue("wavnotrecreate", ui.cB_WavNotReCreate->isChecked());
		settings.setValue("wavdelfiles", ui.cB_WavDelFiles->isChecked());
	settings.endGroup();

	settings.beginGroup("out");
		settings.setValue("outdir", ui.lE_Out->text());
		settings.setValue("outcreatesubdir", ui.cB_OutCreateSubDir->isChecked());
		settings.setValue("outdelspaces", ui.cB_OutDelSpaces->isChecked());
		settings.setValue("outshortento8", ui.cB_OutShorten->isChecked());
		settings.setValue("outrandomize", ui.cB_OutRandomize->isChecked());
		settings.setValue("outtagsreadd", ui.cB_OutTagsReAdd->isChecked());
	settings.endGroup();

	settings.sync();
}

// *****************************************
// **  actions, menu, toolbar, statusbar  **
// *****************************************
void Qntg::actionsCreate()
{
	aboutAct = new QAction(QIcon("/usr/share/normalizetogo/icons/scalable/about.svg"), tr("&About normalizetogo"), this);
	aboutAct->setShortcut(tr("Ctrl+A"));
	aboutAct->setToolTip(tr("Show the application's about information"));
	connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));
	
	runAct = new QAction(QIcon("/usr/share/normalizetogo/icons/scalable/run.svg"), tr("&Create output"), this);
	runAct->setShortcut (tr("Ctrl+C"));
	runAct->setToolTip(tr("Create normalized output"));
	connect(runAct, SIGNAL(triggered()), this, SLOT(runClicked()));

	disabledAct = new QAction(QIcon("/usr/share/normalizetogo/icons/scalable/disabled.svg"), tr("Creating normalized output not possible"), this);
	disabledAct->setToolTip(tr("Creating output not possible - see bottom of screen"));
	disabledAct->setStatusTip(tr("Creating normalized output not possible - wavegain, lame not found - wav and/or output directory invalid - no mp3s in source dir."));

	exitAct = new QAction(QIcon("/usr/share/normalizetogo/icons/scalable/exit.svg"), tr("&Quit the program"), this);
	exitAct->setShortcut(tr("Ctrl+Q"));
	exitAct->setToolTip(tr("Quit the application"));
	connect(exitAct, SIGNAL(triggered()), this, SLOT(quitPrep()));

	helpAct = new QAction(QIcon("/usr/share/normalizetogo/icons/scalable/help.svg"), tr("&Help"), this);
	helpAct->setShortcut(tr("Ctrl+H"));
	helpAct->setToolTip(tr("Show the application's online help"));
	connect(helpAct, SIGNAL(triggered()), this, SLOT(help()));
/*
	refreshAct = new QAction(QIcon("/usr/share/normalizetogo/icons/scalable/refresh.svg"), tr("&Refresh dir view"), this);
	refreshAct->setShortcut(tr("Ctrl+R"));
	refreshAct->setToolTip(tr("Refresh file browser view"));
//	connect(refreshAct, SIGNAL(triggered()), this, SLOT(dmRefresh()));
*/
	stopAct = new QAction(QIcon("/usr/share/normalizetogo/icons/scalable/stop.svg"), tr("St&op normalization process"), this);
	stopAct->setShortcut(tr("Ctrl+S"));
	stopAct->setToolTip(tr("Stop normalization process - currently not implemented !"));
//	connect(stopAct, SIGNAL(triggered()), this, SLOT(clickedStop()));
}

void Qntg::menuBarCreate()
{
	menuBar()->clear();
	fileMenu = menuBar()->addMenu(tr("&File"));
//		fileMenu->addAction(refreshAct);
//		fileMenu->addSeparator();
		fileMenu->addAction(exitAct);
	menuBar()->addSeparator();
	runMenu = menuBar()->addMenu(tr("&Run"));
		if(iRunState == 0) {					// convert not found or output dir invalid
			runMenu->addAction(disabledAct);
		} else if(iRunState == 1) {				// output creation enabled
			runMenu->addAction(runAct);
		} else if(iRunState == 2) {				// output creation running
			runMenu->addAction(stopAct);
		}
	menuBar()->addSeparator();
	helpMenu = menuBar()->addMenu(tr("&Help"));
		helpMenu->addAction(helpAct);
		helpMenu->addAction(aboutAct);
}

void Qntg::toolBarCreate()
{
	toolBar->clear();
	toolBar->addAction(exitAct);
	toolBar->addSeparator();
	if(iRunState == 0) {					// convert not found or output dir invalid
		toolBar->addAction(disabledAct);
	} else if(iRunState == 1) {				// output creation enabled
		toolBar->addAction(runAct);
	} else if(iRunState == 2) {				// output creation running
		toolBar->addAction(stopAct);
	}
//	toolBar->addSeparator();
//	toolBar->addAction(refreshAct);
}

void Qntg::statusBarCreate()
{
	statusBarUpd(tr("Ready"));
}

void Qntg::statusBarUpd(const QString& sD)
{
	statusBar()->showMessage(sD);
}

void Qntg::updMnTlBr()						// update menu and toolbar
{
	menuBarCreate();
	toolBarCreate();
}

// ****************************************
// **  message boxes					 **
// ****************************************
void Qntg::messageBox(int iD, const QString sD, const QString sD2)	// show QMessageBox at cursor position
{
	getCursorPos();
	QMessageBox msgBox;
		msgBox.setWindowTitle(sD);
		if (iD == 1) {
			msgBox.setIcon(QMessageBox::Information);
		} else if (iD == 2) {
			msgBox.setIcon(QMessageBox::Warning);
		} else if (iD == 3) {
			msgBox.setIcon(QMessageBox::Critical);
		} else {
			return;
		}
		msgBox.setText(sD2);
		msgBox.addButton(tr("&OK"), QMessageBox::AcceptRole);
		msgBox.move(iX, iY);
		msgBox.exec();
	if (bQuit == true) {
		quitPrep();
	}
}

void Qntg::getCursorPos()					// get cursor position - iX=x , iY=y
{
	qPos = QCursor::pos();
	iX = qPos.x() - 100;
	if(iX < 0) {
		iX = 0;
	}
	iY = qPos.y() - 175;
	if(iY < 0) {
		iY = 0;
	}
}

// ****************************************
// **  about/help/error widget handling  **
// ****************************************
void Qntg::about()							// about clicked
{
	wCtrl(1);
}

void Qntg::help()							// help clicked
{
	wCtrl(2);
}

void Qntg::ahwCreate()						// create about/help/error dialog widget
{
	if(iWShown == 0) {
		ahw = new aboutHelp();
		connect(this, SIGNAL(sigAhwShow(int)), ahw, SLOT(dlgShow(int)));
		connect(ahw, SIGNAL(sigDelAhw()), this, SLOT(ahwDel()));
	}
}

void Qntg::ahwDel()							// ahw widget was been hidden - delete it
{
	ahw->deleteLater();
	iWShown = 0;
	if(bQuit) {
		quitPrep();
	}
}

// ****************************************
// **  quit								 **
// ****************************************
void Qntg::quitPrep()                       // quit clicked clean up
{
	getCursorPos();
	QMessageBox msgBox;
	msgBox.setWindowTitle(tr("Quit ?"));
	msgBox.setIcon(QMessageBox::Question);
	msgBox.setText(tr("\nSure to quit ?\n"));
	msgBox.addButton(tr("Yes, &Quit"), QMessageBox::AcceptRole);
	msgBox.addButton(tr("No, &Cancel"), QMessageBox::RejectRole);
	msgBox.move(iX, iY);
	if(msgBox.exec() == 0) {
		bQuit = true;
		settingsWrite();
//		tmpDirDel();
		quitDelay();
	}
}

void Qntg::quitDelay()                      // delay to allow clean up - needed ????????????????????
{
	QTimer::singleShot(500, this, SLOT(quitDo()));
}

void Qntg::quitDo()                         // finally quit the program
{
	exit(0);
}

// ****************************************
// **  init								 **
// ****************************************
void Qntg::initDelay()
{
	QTimer::singleShot(200, this, SLOT(init()));
}

void Qntg::init()
{
	progStatClear();
	chkReqProgs();								// check for required programs (lame, wavegain, taglib), we come back if able to run even without taglib, else quit
	runStateUpd(chkDirsOnStart());				// check input and output dirs on start, whether they are still valid
	updMnTlBr();
	fsmCreate();
}

void Qntg::progStatClear()					// clear program status in widget
{
	ui.l_ProgStatus->setText(tr("idle"));
	qPix.load("/usr/share/normalizetogo/icons/scalable/idle.svg", 0);
	ui.l_S1Icon->setPixmap(qPix);
		ui.l_S1XY->clear();
		ui.l_S1Stat->clear();
	ui.l_S2Icon->setPixmap(qPix);
		ui.l_S2XY->clear();
		ui.l_S2Stat->clear();
	ui.l_S3Icon->setPixmap(qPix);
		ui.l_S3XY->clear();
		ui.l_S3Stat->clear();
	ui.l_S4Icon->setPixmap(qPix);
		ui.l_S4XY->clear();
		ui.l_S4Stat->clear();
	ui.l_ProcStep->clear();
	ui.l_ProcFile->clear();
}

void Qntg::chkReqProgs()					// check for required programs (lame, wavegain, taglib)
{
	iProgsFound = rwChkProgs();					// *10 to make modulo work: 0x/1x=lame, 0xx/1xx=wavegain, 0xxx/1xxx=taglib
	iErr = 0;
	sErr = "";
	if (iProgsFound / 100 % 10 == 0) {			// taglib not found
		iErr = 1;
		bTaglib = false;
	} else {
		ui.cB_Taglib->setChecked(true);
	}
	if (iProgsFound % 10 == 0) {
		iErr += 2;
		bQuit = true;
		sErr = tr("- lame not found");
	} else {
		ui.cB_Lame->setChecked(true);
	}
	if (iProgsFound / 10 % 10 == 0) {
		iErr += 4;
		bQuit = true;
		sErr = tr("- wavegain not found");
	} else {
		ui.cB_Wavegain->setChecked(true);
	}
	if (iErr > 0) {
		if (iErr < 2) {
			messageBox(2, tr("Taglib not found!"), tr("Tags cannot be re-created until taglib has been installed!"));
		} else if (iErr < 8) {
			messageBox(3, tr("Fatal error - program not found!"), tr("Important program(s) not found:\n"
																		"%1\n"
																		"normalizetogo will now end!").arg(sErr));
			bQuit = true;
			quitDo();							// exit if we cannot work
		}
	}
}

int Qntg::chkDirsOnStart()					// check input and output dirs on start, whether they are still valid
{
	iD = 10;

	slD = rwFilesGet(sSrcDir, 1);							// check whether there are any mp3s in src dir
	if (slD.count() == 0) {
		messageBox(1, tr("No mp3s in src dir"), tr("There are no mp3s in the source directory (step 1)!\n%1").arg(sSrcDir));
		iD = 5;
	}
	
	if (rwDirChkWritable(ui.lE_Wav->text()) == false) {		// check whether wav dir is writable, full dir, but sub dir might not yet be created!
		if (ui.cB_WavCreateSubDir->isChecked() == true) {	// if sub dir for wav shall be created during run, check whether we can write to dir 1 level up
			QDir dir(ui.lE_Wav->text());
				dir.cdUp();
			if (rwDirChkWritable(dir.absolutePath()) == false) {	// cannot create the wav sub-dir when running - now we are in trouble
				messageBox(1, tr("wav dir not writable"), tr("The wav directory is not writable (step 2)!\n%1").arg(dir.absolutePath()));
				iD = 5;
			}
		} else {
			messageBox(1, tr("wav dir not writable"), tr("The wav directory is not writable (step 2)!\n%1").arg(ui.lE_Wav->text()));
			iD = 5;
		}
	}
	
	if (rwDirChkWritable(ui.lE_Out->text()) == false) {		// check whether output dir is writable, full dir, but sub dir might not yet be created!
		if (ui.cB_OutCreateSubDir->isChecked() == true) {	// if sub dir for output shall be created during run, check whether we can write to dir 1 level up
			QDir dir(ui.lE_Out->text());
				dir.cdUp();
//cout << "dir.absolutePath() = " << dir.absolutePath().toStdString() << endl;      //  *******************  debug  !!!!!!!!!!
			if (rwDirChkWritable(dir.absolutePath()) == false) {	// cannot create the wav sub-dir when running - now we are in trouble
				messageBox(1, tr("Out dir not writable"), tr("The output directory is not writable (step 3)!\n%1").arg(dir.absolutePath()));
				iD = 5;
			}
		} else {
			messageBox(1, tr("Out dir not writable"), tr("The output directory is not writable (step 2)!\n%1").arg(ui.lE_Out->text()));
			iD = 5;
		}
	}
	return iD;
}

// ***********************************************************
// **  source file system model functions					**
// ***********************************************************
void Qntg::fsmCreate()
{
//cout << "fsmCreate() : reached " << endl;      //  *******************  debug  !!!!!!!!!!
	statusBarUpd(tr("Just a moment, creating directory tree..."));

	fsModel = new QFileSystemModel();
		fsModel->setRootPath("/");
	ui.tV_FileSys->setModel(fsModel);
//		ui.tV_FileSys->sortByColumn (0, Qt::SortOrder(Qt::AscendingOrder));
		ui.tV_FileSys->setSelectionBehavior(QAbstractItemView::SelectionBehavior(QAbstractItemView::SelectItems));
		ui.tV_FileSys->setSelectionMode(QAbstractItemView::SelectionMode(QAbstractItemView::SingleSelection));
		ui.tV_FileSys->header()->setSectionResizeMode(QHeaderView::Interactive);
		ui.tV_FileSys->setColumnWidth(0, iTvCw0);	// set column width
		ui.tV_FileSys->setColumnWidth(1, iTvCw1);
		ui.tV_FileSys->setColumnWidth(2, iTvCw2);
		ui.tV_FileSys->setColumnWidth(3, iTvCw3);
		//	connect(ui.tV_FileSys, SIGNAL(expanded(const QModelIndex &)), this, SLOT(fsmAutoResizeColumns(const QModelIndex &)));
		//	connect(ui.tV_FileSys, SIGNAL(activated(const QModelIndex &)), this, SLOT(fsmSelectionChanged(const QModelIndex &)));
		connect(ui.tV_FileSys, SIGNAL(clicked(const QModelIndex &)), this, SLOT(fsmSelectionChanged(const QModelIndex &)));
	fsmExpandToLastDir();

	statusBarUpd(tr("Ready"));
}

void Qntg::fsmExpandToLastDir()				// expand to dir used in last session
{
	QDir dir(sSrcDir);
	do {
		if (dir.exists() == true) {				// old dir still exists, expand
			miIndex = fsModel->index(dir.absolutePath(), 0);
			ui.tV_FileSys->expand(miIndex);
			ui.tV_FileSys->setCurrentIndex(miIndex);
			break;
		}
	} while (dir.cdUp() == true);
}

void Qntg::fsmSelectionChanged(const QModelIndex& miD)	// selection in tree view has changed
{
//cout << "fsmSelectionChanged() : reached " << endl;				//  *******************  debug  !!!!!!!!!!
	if (fsModel->fileInfo(miD).isFile() == true) {			// get path of selected item in tree view
		sSrcDir = fsModel->fileInfo(miD).path();
	} else {
		sSrcDir = fsModel->filePath(miD);
	}
	sSrcDir = rwDirPathEnsureSlash(sSrcDir);				// ensure, that it ends on a '/'

//cout << "fsmSelectionChanged() : sSrcDir = " << sSrcDir.toStdString() << endl;      //  *******************  debug  !!!!!!!!!!
	ui.tV_FileSys->expand(miD);

	iRunState = 0;											// before check, disable normalization run
	if (rwFilesGet(sSrcDir, 1).count() > 0) {				// check if any mp3s in dir
		if (ui.cB_WavCreateSubDir->isChecked() == true) {	// can we handle wavs in self created sub dir?
			if (rwDirChkWritable(sSrcDir) == true) {				// check if writeable
				ui.lE_Wav->setText(rwDirSubSetName(sSrcDir, 1));
				iRunState = 1;
			}
		}

		if (ui.cB_OutCreateSubDir->isChecked() == true) {	// can we handle mp3s in self created sub dir?
			if (rwDirChkWritable(sSrcDir) == true) {				// check if writeable
				ui.lE_Out->setText(rwDirSubSetName(sSrcDir, 2));
				iRunState = 1;
			}
		}
	} else {
		// disable start icon, nothing to do!!
	}

	updMnTlBr();											// update GUI
}

void Qntg::runStateUpd(int iD)				// update run state n menu n toolbar: iD: 0=lame missing, 1=wavegain missing, 2=tool for m4a missing
{																				//	  5=no file in dir, 10=idle, 20=running
	if (iD < 10) {
		iRunState = 0;							//	  run states: 0=cannot create
	} else if (iD < 20) {
		iRunState = 1;							//	  1=can create
	} else {
		iRunState = 2;							//	  2=running, can stop(kill)
	}
	updMnTlBr();
}

// ***********************************************************
// **  wav dir handling										**
// ***********************************************************
void Qntg::destDirSetClicked()				// icon to set destination dir clicked
{
  destDirGetStartDir();

  sD = "";
  QFileDialog fd(this);
    fd.setFileMode(QFileDialog::Directory);
    fd.setViewMode(QFileDialog::List);
    fd.setNameFilter(tr("ALL (*.*)"));
    fd.setDirectory(sDir);
    if(fd.exec()) {
		slD = fd.selectedFiles();
		sOutDir = destDirStripDir(slD.at(0));
		destDirSet(sOutDir);
    }
}

void Qntg::destDirEditFin()					// manually setting the pdf path finished
{
	destDirSet(destDirStripDir(ui.lE_Out->text()));
}

void Qntg::destDirSet(const QString& sD)	// set dest dir in line widget, home dir if invalid
{
	if(rwDirChkWritable(sD) == true) {
		sOutDir = rwDirPathEnsureSlash(sOutDir);
	} else {
		messageBox(2, tr("Error"), tr("The selected directory\n%1\nis not writable!\nOutput dir set to your home directory.").arg(sOutDir));
		sOutDir = rwDirPathEnsureSlash(QDir::homePath());
	}
	ui.lE_Out->setText(sOutDir);
}

QString Qntg::destDirStripDir(const QString& sD)	// get path from dir or file
{
	QFileInfo fi2(sD);
	if(fi2.isDir() == true) {
		QDir di(sD);
		return di.absolutePath();
	} else {
		return fi2.absolutePath();
	}
}

void Qntg::destDirGetStartDir()				// get start dir for set dir dialog
{
	sDir = ui.lE_Out->text();
	QFileInfo fi(sDir);						// get start dir for dialog
	if(fi.isDir() == false) {
		sDir = fi.absolutePath();
	}
	QFileInfo fi2(sDir);
	if(fi2.exists() == false or fi2.isWritable() == false) {
		sDir = QDir::homePath();
	}
}

void Qntg::wavCreateSubDirToggled()			// create wav sub dir checkbox toggled
{
	if (ui.cB_WavCreateSubDir->isChecked() == true) {
		ui.lE_Wav->setEnabled(false);
		ui.pB_Wav->setEnabled(false);
		ui.lE_Wav->setText(rwDirSubSetName(sSrcDir, 1));
	} else {
		ui.lE_Wav->setEnabled(true);
		ui.pB_Wav->setEnabled(true);
	}
}

void Qntg::outCreateSubDirToggled()			// create output sub dir checkbox toggled
{
	if (ui.cB_OutCreateSubDir->isChecked() == true) {
		ui.lE_Out->setEnabled(false);
		ui.pB_Out->setEnabled(false);
		ui.lE_Out->setText(rwDirSubSetName(sSrcDir, 2));
	} else {
		ui.lE_Out->setEnabled(true);
		ui.pB_Out->setEnabled(true);
	}
}

void Qntg::showErr()						// show any error during pdf creation
{
	if(iErr > 0) {                              // only proceed, if no other widget is shown
		switch(iErr) {
			case 1:                                   // output dir not OK
				messageBox(2, tr("Error - output dir not OK !"), tr("Problem with output directory !\nDoes it exist, is it writable?\nAborting..."));
				break;
//			case 2:                                   // no file selected, must never happen, create button should be disabled in this case !
//				messageBox(2, tr("Error - nothing to do !"), tr("No file is selected - nothing to do.\nThis should never happen.\nAborting..."));
//				break;
			case 8:                                   // error concatenating every 2 pics together, left and right
				messageBox(2, tr("Error: concatenating pictures!"), sErr8);
				break;
			case 32:                                   // error while creating pdf with PDFlib lite
				messageBox(2, tr("Error: pdf creation aborted!"), sErr32);
				break;
    }
  }
}

//  **  create full output file path & name
QString Qntg::outputSetOutputFile()
{
	sOutFullName = "";
/*
	if(ui.cB_InSrcDir->isChecked() == true) {		// set source dir as output dir
		QFileInfo fi(slPicsSel.at(0));
		sOutFullName = fi.absolutePath();
	} else if(ui.lE_Path->text().length() > 0) {	// add output path from lineedit
		sOutFullName += ui.lE_Wav->text();
	} else {										// empty output path
		messageBox(1, tr("Error"), tr("Path of output file could not be set!\nAborting..."));
		return "";
	}
*/
	if(sOutFullName.right(1) != "/") {				// ensure, that we have / at end of path
		sOutFullName += "/";
	}
/*
	if(ui.lE_File->text().length() > 0) {			// add output file name, use suffic as is, might be user overwritten
		sOutFullName += ui.lE_File->text();
		sD = sOutFullName.right(4);
		if(bPdf == true and sD.contains(".pdf", Qt::CaseInsensitive) == false) {
			sOutFullName += sOutFullName + ".pdf";
		} else if(bPdf == false and sD.contains(".jpg", Qt::CaseInsensitive) == false ) {
			sOutFullName += sOutFullName + ".jpg";
		}
		return sOutFullName;
	} else {
		messageBox(1, tr("Error"), tr("The output file name could not be set!\nAborting..."));
		return "";									// empty output file name
	}
*/
}

//  **  check whether output file can be writen in destination dir and whether to overwrite file if file already exists
bool Qntg::outputChkOutputFile()
{
/*
	if (ui.cB_InSrcDir->isChecked() == true) {		// check if source/destination dir is writeable
		sD = slPicsSel.at(0);
		sErr = tr("source");
	} else {
		sD = sOutFullName;
		sErr = tr("destination");
	}
*/
	if(rwDirChkWritable(sD) == false) {
		messageBox(2, tr("Error"), tr("Cannot create output file in %1 directory\n%2\nIt is not writeable.\nAborting...").arg(sErr).arg(fi.absolutePath()));
		return false;
	}

	QFileInfo fi(sOutFullName);						// check if destination file already exists
	if (fi.exists() == true) {
//		disconnect(ui.lE_File, SIGNAL(editingFinished()), this, SLOT(outNameLeEditFin()));		
		getCursorPos();
		QMessageBox msgBox;
		msgBox.setWindowTitle(tr("destination file exists !"));
		msgBox.setIcon(QMessageBox::Question);
		msgBox.setText(tr("\nThe destination file\n%1\nalready exists !\nDo you want to overwrite it?").arg(sOutFullName));
		msgBox.addButton(tr("&Yes, Overwrite"), QMessageBox::AcceptRole);
		msgBox.addButton(tr("No, &Cancel"), QMessageBox::RejectRole);
		msgBox.move(iX, iY);
		if (msgBox.exec() == QMessageBox::AcceptRole) {
//			connect(ui.lE_File, SIGNAL(editingFinished()), this, SLOT(outNameLeEditFin()));
			return true;							// continue with this path/file name - we want to overwrite
		} else {
//			connect(ui.lE_File, SIGNAL(editingFinished()), this, SLOT(outNameLeEditFin()));
			return false;							// abort
		}
	} else {
		return true;								// does not yet exist, continue
	}
}

void Qntg::outputSetType()					// set output type, pdf or jpg
{
//	bPdf = ui.rB_Pdf->isChecked();
}

void Qntg::outputCreatePrep()				// prepare output creation
{
	iRunState = 2;
	updMnTlBr();

//  ** create pdf
/*	if (bPdf == true) {
		if (ui.cB_PdfConcPic->isChecked() == true and slPicsSel.count() > 1) {
//cout << "pdfCreatePrep() : creating tmpDir " << sTmpDir.toStdString() << endl;      //  *******************  debug  !!!!!!!!!!
			tmpDirCreate();						// create temp dir
			magickAppend();
		} else {								// create pdf directly from selection, no concatenate
			slPicsToPdf = slPicsSel;
			QFileInfo fi(slPicsToPdf.at(0));
			sTmpDir = "";						// empty dir = provided pics have full path n name (= selection)
			magickToPdf();
		}

// ** create jpg
	} else {
//		magickAppend();
	}*/
}


// ***********************************************************
// **  normalization run functions							**
// ***********************************************************
void Qntg::runClicked()						// create output file clicked
{
	iDirCreatedWav = 2;
	iDirCreatedOut = 2;
	bSuccess = false;							// flag whether the process went somewhat through (produced at least some normalized output files)
	iErr = 0;
	iRunLevel = 0;
	runGuiUpdRunLevel();
	iRunState = 2;								// run state: 0=cannot create, 1=can create, 2=running, can stop(kill)
	updMnTlBr();
	ui.l_ProgStatus->setText(tr("running..."));
	runBegin();
}

void Qntg::runBegin()						// run begin, create dirs, get mp3s to process
{
	if (run1DirsCreate() != 1) {				// create dirs
		runFailed();
		return;										// failed, no need to clean up ???
	}

	if (run2GetMp3s() != 1) {					// get mp3s to process
		runFailed();
		return;
	}

	run3LameDecodeToWav();					// decode mp3s to wav
}

bool Qntg::run1DirsCreate()					// create wav and normalized output directory
{
//  **  create wav dir
	iRunLevel = 1;
	runGuiUpdRunLevel();
	if (ui.cB_WavCreateSubDir->isChecked() == true) {
		iDirCreatedWav = rwDirCreate(ui.lE_Wav->text());
		if (iDirCreatedWav == 0) {
			messageBox(2, tr("Error creating directory"), tr("Could not create the directory\n%1").arg(ui.lE_Wav->text()));
			return 0;
		}
	}

//  **  create dir for normalized output
	if (ui.cB_OutCreateSubDir->isChecked() == true) {
		iDirCreatedOut = rwDirCreate(ui.lE_Out->text());
		if (iDirCreatedOut == 0) {
			messageBox(2, tr("Error creating directory"), tr("Could not create the directory\n%1").arg(ui.lE_Out->text()));
			return 0;
		}
	}
	return 1;
}

bool Qntg::run2GetMp3s()					// get list of mp3s to normalize
{
	iRunLevel = 2;
	runGuiUpdRunLevel();
	slMp3sToProc = rwFilesGet(sSrcDir, 1);
		if (slMp3sToProc.count() == 0) {
			messageBox(2, tr("Error getting mp3s"), tr("Could not find any mp3s to process in the directory\n%1").arg(sSrcDir));
			return 0;
		}
	return 1;
}

void Qntg::run3LameDecodeToWav()			// decode mp3s to wav with lame
{
	iRunLevel = 3;
	runGuiUpdRunLevel();
//cout << "run3LameDecodeToWav() : 1st entry = " << slMp3sToProc.at(0).toStdString() << endl;      //  *******************  debug  !!!!!!!!!!
	emit sigLameDo(sSrcDir, slMp3sToProc, ui.lE_Wav->text(), 1, ui.cB_WavNotReCreate->isChecked(), false, false);		// 1=decode to wav, last 2 bool ignored
}

void Qntg::run3LameDecodeToWavReturn(const QStringList& slD, const QList<FilePairs>& liFilePairsD)	// return from decode mp3s to wav with lame
{
	slErrors = slD;								// files with errors - could not decode
//cout << "run3LameDecodeToWavReturn() : slErrors count = " << QString::number(slErrors.count()).toStdString() << endl;
	liFilePairsMp3Wav = liFilePairsD;			// valid mp3 to wav pairs
//cout << "run3LameDecodeToWavReturn() : liFilePairsMp3Wav count = " << QString::number(liFilePairsMp3Wav.count()).toStdString() << endl;
	if (slErrors.count() > 0) {					// errors
		if (liFilePairsMp3Wav.count() > 0) {		// we have encoded some files successfully, but not all - shall we stop or continue?
			QMessageBox msgBox;
				msgBox.setText("Some mp3s could not be encoded to wav");
				msgBox.setInformativeText("Do you want to continue?");
				msgBox.setDetailedText(slErrors.join("\n"));
				msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
				msgBox.setDefaultButton(QMessageBox::Yes);
				ret = msgBox.exec();
			if (ret == QMessageBox::No) {			// abort
				runFailed();
				return;
			}
		} else {									// encoding to wav failed completely - no way to continue
			QMessageBox msgBox;
				msgBox.setText("No mp3 could be encoded to wav - aborting!");
				msgBox.exec();
			runFailed();
			return;
		}
	}
//cout << "run3LameDecodeToWavReturn() : 1st entry = " << slErrors.at(0).toStdString() << endl;      //  *******************  debug  !!!!!!!!!!
	run4WavegainNormalize();
}

void Qntg::run4WavegainNormalize()			// normalize wavs in dir
{
	iRunLevel = 4;
	runGuiUpdRunLevel();
	emit sigWavegainStart(ui.lE_Wav->text());
}

void Qntg::run4WavegainNormalizeReturn(bool bD)	// return from normalize wavs in dir
{
	if (bD == false) {									// errors
		cout << "run4WavegainNormalizeReturn() : wavegain ended with errors - need to show this in a pop up ! Stopping!" << endl;
		runFailed();
		return;										//   ************* need a clean up function here ????????????????????????????????????
	}
	run5EncodeWavToMp3();
}

void Qntg::run5EncodeWavToMp3()				// encode wavs back to mp3s
{
//cout << "run5EncodeWavToMp3() : reached!" << endl;
	iRunLevel = 5;
	runGuiUpdRunLevel();
	slWavsToProc = rwFilesGet(ui.lE_Wav->text(), 2);
		if (slWavsToProc.count() == 0) {
			cout << "run5EncodeWavToMp3() : slWavsToProc.count()=0 !" << endl;
			messageBox(2, tr("Error getting wavs"), tr("Could not find any wavs to process in the directory\n%1\nAborting...").arg(ui.lE_Wav->text()));
			runFailed();
			return;
		}
	emit sigLameDo(ui.lE_Wav->text(), slWavsToProc, ui.lE_Out->text(), 2, ui.cB_OutRandomize->isChecked(), ui.cB_OutDelSpaces->isChecked(), ui.cB_OutShorten->isChecked());
						// send src dir, file list with mp3s/wavs to process (name), output dir, command 1=decode/2=encode, randomize, del spaces, shorten to 8
}

void Qntg::run5EncodeWavToMp3Return(const QStringList& slD, const QList<FilePairs>& liFilePairsD)	// return from encode wavs back to mp3s
{
	slErrors = slD;
	liFilePairsWavMp3 = liFilePairsD;			// list struct with file name pairs successfully encoded from wav to to mp3 (normalized)
	if (liFilePairsWavMp3.count() == 0) {		// epic fail - no normalized mp3 created!
		QMessageBox msgBox;
			msgBox.setText(tr("No wav could be encoded to mp3 - aborting!"));
			msgBox.exec();
		runFailed();							// we finished unsuccessfully !
		return;
	} else if (slErrors.count() > 0) {			// some errors
		QMessageBox msgBox;
			msgBox.setText(tr("Some wavs could not be encoded to mp3"));
			msgBox.setInformativeText(tr("Do you want to continue?"));
			msgBox.setDetailedText(slErrors.join("\n"));
			msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
			msgBox.setDefaultButton(QMessageBox::Yes);
		ret = msgBox.exec();
		if (ret == QMessageBox::No) {			// abort
			runFailed();
			return;
		}
	}

	if (ui.cB_OutTagsReAdd->isChecked() == true) {
		// get pairs of mp3s orig and normalized worth trying to re-add tags
		if (liFilePairsMp3Wav.count() > 0 and liFilePairsWavMp3.count() > 0) {
//cout << "run5EncodeWavToMp3Return() : liFilePairsMp3Wav count = " << QString::number(liFilePairsMp3Wav.count()).toStdString() << " , liFilePairsWavMp3 count = " << QString::number(liFilePairsWavMp3.count()).toStdString() << endl;
			liFilePairsReTag.clear();
			for (int i = 0; i < liFilePairsMp3Wav.count(); i++) {
				for (int i2 = 0; i2 < liFilePairsWavMp3.count(); i2++) {
					if (liFilePairsMp3Wav.at(i).out == liFilePairsWavMp3.at(i2).src) {
						fpD.src = liFilePairsMp3Wav.at(i).src;
						fpD.out = liFilePairsWavMp3.at(i2).out;
						liFilePairsReTag.append(fpD);
					}
				}
			}
			run7ReAddTags();						// continue and re-add tags
		} else {
//cout << "run5EncodeWavToMp3Return() : liFilePairsMp3Wav count = 0 " << QString::number(liFilePairsMp3Wav.count()).toStdString() << " , liFilePairsWavMp3 count = 0 " << QString::number(liFilePairsWavMp3.count()).toStdString() << endl;
			iRunLevel = 6;							// 6=6FinishedNoTags
			runGuiUpdRunLevel();
			runFailed();							// we finished unsuccessfully !
		}
	} else {
		iRunLevel = 6;							// 6=6FinishedNoTags
		runGuiUpdRunLevel();
		runSuccess();							// we finished successfully !
	}
}

void Qntg::run7ReAddTags()					// re-add tags with taglib
{
//cout << "run7ReAddTags() : liFilePairsReTag.count() = " << QString::number(liFilePairsReTag.count()).toStdString() << endl;
	iRunLevel = 7;
	runGuiUpdRunLevel();
	emit sigTaglibReAdd(liFilePairsReTag);		// re-add tags, list with pairs of full path and name of original and normalized files
}

void Qntg::run7ReAddTagsReturn(const QStringList& slD)	// return from re-add tags with taglib - list with files of which tags could not be re-added
{
	slErrors = slD;

	if (slErrors.count() > 0) {					// some errors
		if (liFilePairsReTag.count() == slErrors.count()) {
			sD = tr("all");							// could not re-add any tags
		} else {
			sD = tr("some");						// could at least some tags, if not all
		}
		QMessageBox msgBox;
			msgBox.setText(tr("Errors encountered while re-adding tags."));
			msgBox.setInformativeText(tr("Re-adding tags failed on %1 files.").arg(sD));
			msgBox.setDetailedText(slErrors.join("\n"));
			msgBox.setStandardButtons(QMessageBox::Ok);
			msgBox.setDefaultButton(QMessageBox::Ok);
		ret = msgBox.exec();
	}
//cout << "run7ReAddTagsReturn() : liFilePairsReTag.count() = " << QString::number(liFilePairsReTag.count()).toStdString() << "   slErrors.count() = " << QString::number(slErrors.count()).toStdString() << endl;
	iRunLevel = 8;								// 8=8FinishedAll
	runGuiUpdRunLevel();
	runSuccess();
}

void Qntg::runSuccess()						// run was successful - re-set Gui and states - runlevel 6=finished no tags / 8=finished re-adding tags
{
	// clean up if anything to do - delete files if asked for
	if (ui.cB_WavDelSubDir->isChecked() == true or ui.cB_WavDelFiles->isChecked() == true) {
		if (run3DelFilesWav() == 0) {
			messageBox(2, tr("Error(s) while deleting wav files"), tr("Errors were encountered while trying to delete wav files.\n"
																		"Please check and clean the wav directory manually."));
		} else if (ui.cB_WavDelSubDir->isChecked()) {
			if (run1DelDirWav() == 0) {					// error deleting created wav dir
				messageBox(2, tr("Error while deleting wav directory"), tr("Errors were encountered while trying to delete the wav directory.\n"
																			"Please check and delete the wav directory manually."));
			}
		}
	}

	// re-set everything (GUI)
	iRunState = 1;
	updMnTlBr();
	iRunLevel = 0;
	runGuiUpdRunLevel();

	sD2 = tr("Normalization ended successfully");	// info missing here
	messageBox(1, tr("Normalization ended successfully"), sD2);		// did we track any errors? if yes, show them.
}

void Qntg::runFailed()						// run failed - clean up, re-set Gui and states
{
	if (iRunLevel > 4) {						// delete created mp3 files in out dir
		if (run5DelFilesMp3() == 0) {				// error deleting created wav files
			messageBox(2, tr("Error(s) while deleting mp3 files"), tr("Errors were encountered while trying to delete mp3 files.\n"
																		"Please check and clean the output directory manually."));
		}
	}

	if (iRunLevel > 2) {						// delete created wav files in wav dir
		if (run3DelFilesWav() == 0) {				// error deleting created wav files
			messageBox(2, tr("Error(s) while deleting wav files"), tr("Errors were encountered while trying to delete wav files.\n"
																		"Please check and clean the wav directory manually."));
		}
	}

	if (iRunLevel > 0) {						// delete wav and out dir if created by us (need to be empty to succeed)
		if (run1DelDirWav() == 0) {					// error deleting created wav dir
			messageBox(2, tr("Error while deleting wav directory"), tr("Errors were encountered while trying to delete the wav directory.\n"
																		"Please check and delete the wav directory manually."));
		}
		if (run1DelDirOut() == 0) {					// error deleting created wav dir
			messageBox(2, tr("Error while deleting output directory"), tr("Errors were encountered while trying to delete the output directory.\n"
																		"Please check and delete the output directory manually."));
		}
	}

	// re-set everything (GUI)
	iRunState = 1;
	updMnTlBr();
	iRunLevel = 0;
	runGuiUpdRunLevel();

	sD2 = tr("Normalization ended successfully");	// info missing here
	messageBox(1, tr("Normalization ended successfully"), sD2);		// did we track any errors? if yes, show them.
}

bool Qntg::run1DelDirWav()						// try to delete wav directory if we created it
{
	if (iDirCreatedWav == 1) {						// do not delete a dir we did not created
		return rwDirDel(ui.lE_Wav->text());
	}
}

bool Qntg::run1DelDirOut()						// try to delete out directory if we created it
{
	if (iDirCreatedOut == 1) {						// do not delete a dir we did not created
		return rwDirDel(ui.lE_Out->text());
	}
}

bool Qntg::run3DelFilesWav()					// try to delete created wav files in wav directory
{
	if (liFilePairsMp3Wav.count() > 0) {				// only delete what we created
		slD.clear();
		for (int i = 0; i < liFilePairsMp3Wav.size(); ++i) {
			fpD = liFilePairsMp3Wav.at(i);
			slD.append(fpD.out);
		}
		return rwFilesDel(slD);
	} else {
		return 1;
	}
}

bool Qntg::run5DelFilesMp3()					// try to delete created mp3 files in output directory
{
	if (liFilePairsMp3Wav.count() > 0) {				// only delete what we created
		slD.clear();
		for (int i = 0; i < liFilePairsMp3Wav.size(); ++i) {
			fpD = liFilePairsMp3Wav.at(i);
			slD.append(fpD.out);
		}
		return rwFilesDel(slD);
	} else {
		return 1;
	}
}

void Qntg::runGuiUpdFileProc(const QString& sD)	// update GUI - file currently processed
{
	ui.l_ProcFile->setText(sD);
}

void Qntg::runGuiUpdRunLevel()				// update GUI according to current runlevel
{
	switch(iRunLevel) {
		case 0:									// all reset
			ui.l_ProgStatus->setText(tr("idle"));
			qPix.load("/usr/share/normalizetogo/icons/scalable/idle.svg", 0);
			ui.l_S1Icon->setPixmap(qPix);
				ui.l_S1XY->clear();
				ui.l_S1Stat->clear();
			ui.l_S2Icon->setPixmap(qPix);
				ui.l_S2XY->clear();
				ui.l_S2Stat->clear();
			ui.l_S3Icon->setPixmap(qPix);
				ui.l_S3XY->clear();
				ui.l_S3Stat->clear();
			ui.l_S4Icon->setPixmap(qPix);
				ui.l_S4XY->clear();
				ui.l_S4Stat->clear();
			ui.l_ProcStep->clear();
			ui.l_ProcFile->clear();
			break;
		case 1:									// 1=1DirsCreate
			ui.l_ProgStatus->setText(tr("running..."));
			qPix.load("/usr/share/normalizetogo/icons/scalable/task-due.svg", 0);
			ui.l_S1Icon->setPixmap(qPix);
			ui.l_S2Icon->setPixmap(qPix);
			ui.l_S3Icon->setPixmap(qPix);
			if (ui.cB_OutTagsReAdd->isChecked() == true) {
				ui.l_S4Icon->setPixmap(qPix);
			}
			ui.l_ProcStep->setText("prep...");
			ui.l_ProcFile->setText(tr("creating directories..."));
			break;
		case 2:									// 2=2GetMp3s
			ui.l_ProgStatus->setText(tr("running..."));
			ui.l_ProcStep->setText("prep...2");
			ui.l_ProcFile->setText(tr("getting mp3s to process..."));
			break;
		case 3:									// 3=3LameDecodeToWav
			ui.l_S1Stat->setText(tr("in progress..."));
			ui.l_ProcStep->setText("1");
			ui.l_ProcFile->setText(tr("decoding to wav..."));				// need files to show according to progress !!!!!
			break;
		case 4:									// 4=4WavegainNormalize
			qPix.load("/usr/share/normalizetogo/icons/scalable/idle.svg", 0);
			ui.l_S1Icon->setPixmap(qPix);
			qPix.load("/usr/share/normalizetogo/icons/scalable/ok.svg", 0);
			qPix = qPix.scaled(48, 48, Qt::IgnoreAspectRatio, Qt::FastTransformation);
			ui.l_S1Stat->setPixmap(qPix);
			ui.l_S2Stat->setText(tr("in progress..."));
			ui.l_ProcStep->setText("2");
			ui.l_ProcFile->setText(tr("normalizing wavs..."));				// need files to show according to progress !!!!!
			break;
		case 5:									// 5=5EncodeWavToMp3
			ui.l_S2Stat->setPixmap(qPix);			// now ok.svg
			qPix.load("/usr/share/normalizetogo/icons/scalable/idle.svg", 0);
			ui.l_S2Icon->setPixmap(qPix);
			ui.l_S3Stat->setText(tr("in progress..."));
			ui.l_ProcStep->setText("3");
			ui.l_ProcFile->setText(tr("re-encoding to mp3..."));			// need files to show according to progress !!!!!
			break;
		case 6:									// 6=6update GUI when not re-adding tags
			ui.l_S3Icon->setPixmap(qPix);			// now idle.svg
			qPix.load("/usr/share/normalizetogo/icons/scalable/ok.svg", 0);
			qPix = qPix.scaled(48, 48, Qt::IgnoreAspectRatio, Qt::FastTransformation);
			ui.l_S3Stat->setPixmap(qPix);
			ui.l_ProcStep->setText("");
			ui.l_ProcFile->setText("");
			break;
		case 7:									// 7=7re-adding tags (comming from 5, 6 was not executed)
			ui.l_S3Icon->setPixmap(qPix);			// now idle.svg
			qPix.load("/usr/share/normalizetogo/icons/scalable/ok.svg", 0);
			qPix = qPix.scaled(48, 48, Qt::IgnoreAspectRatio, Qt::FastTransformation);
			ui.l_S3Stat->setPixmap(qPix);			// now ok.svg
			ui.l_S4Stat->setText(tr("in progress..."));
			ui.l_ProcStep->setText("4");
			ui.l_ProcFile->setText(tr("re-adding tags..."));				// need files to show according to progress !!!!!
			break;
		case 8:									// 8=8finished re-adding tags
			ui.l_S4Stat->setPixmap(qPix);			// now idle.svg
			qPix.load("/usr/share/normalizetogo/icons/scalable/idle.svg", 0);
			qPix = qPix.scaled(48, 48, Qt::IgnoreAspectRatio, Qt::FastTransformation);
			ui.l_S4Icon->setPixmap(qPix);			// now ok.svg
			ui.l_ProcStep->setText("");
			ui.l_ProcFile->setText("");
			ui.l_ProgStatus->setText(tr("finished"));
			break;
	}
}

// ********************************************************************************************************************
// **  popup widget handling        **
// ***********************************
void Qntg::wCtrl(int iWToShow)	// show popup widget, if no other is shown: 0=none, 1=about-abouthelp, 2=help-abouthelp, 3=error-abouthelp
{
	if(iWShown == 0) {						// only proceed, if no other widget is shown
		switch(iWToShow) {
			case 1:								// show about widget
				ahwCreate();
				iWShown = 1;
				emit sigAhwShow(iWShown);
				break;
			case 2:								// show help widget
				ahwCreate();
				iWShown = 2;
				emit sigAhwShow(iWShown);
				break;
		}
	}
}


#include "ntg.moc"
