/****************************************************************************
 **
 **  Copyright (C) 2015-2016 by Helge Kortendieck, alan@kasmithee.de
 **
 **  This file may be used under the terms of the GNU General Public
 **  License version 2.0 or later as published by the Free Software Foundation
 **
 **  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 **  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/

#ifndef _NTG_H_
#define _NTG_H_

#include <QApplication>
#include <QAbstractItemView>
#include <QButtonGroup>
#include <QDialog>
#include <QDir>
#include <QFile>
#include <QFileDialog>
#include <QFileInfo>
#include <QFileSystemModel>
#include <QItemSelectionModel>	// needed?
#include <QList>
#include <QMainWindow>
#include <QMenuBar>
#include <QMessageBox>
#include <QModelIndex>
#include <QPicture>			// needed for scaling icons??
#include <QPoint>
#include <QSettings>
#include <QSizePolicy>
//#include <QSpinBox>
#include <QString>
#include <QStringList>
#include <QTimer>
#include <QToolBar>
#include <QTreeView>
#include <QWidget>

#include "abouthelp.h"
#include "lame.h"
#include "rw.h"
#include "structs.h"
#include "taglib.h"
#include "wavegain.h"

#include "ui_ntg.h"

//#include <cstdlib>
#include <iostream>
using namespace std;

class Qntg : public QMainWindow
{
  Q_OBJECT

public:
	Qntg(QMainWindow *parent = 0);
		~Qntg();

private:
	Ui::Wntg ui;			// GUI

	bool bD;
	bool bD2;
	bool bD3;
	bool bQuit;				// flag whether we are quitting
	bool bSuccess;			// flag whether the process went somewhat through (produced at least some normalized output files)
	bool bTaglib;			// flag have taglib or not

	double dD;
	double dD2;
	double dDpiX;
	double dDpiY;

	int iD;					// dummy int
	int iDirCreatedWav;		// who created wav dir? 0=we failed, 1=we, need to clean up eventually, 2=already existed - do not clean up
	int iDirCreatedOut;		// who created output dir? 0=we failed, 1=we, need to clean up eventually, 2=already existed - do not clean up
	int iErr;				// error code: 0=no error, 1=output dir not OK, 2=nothing selected, 4=error reading images, 8=failed to create tmp dir, 
							//             16=error concatenate 2 images into 1, 32=error creating pdf file, 64=error deleting tmp dir
	int iLen;				// length of file name to suggest pdf name from
	int iRunLevel;			// run level during process run, 1 to 8, 1=1DirsCreate, 	2=2GetMp3s,			3=3LameDecodeToWav,	4=4WavegainNormalize,
																//	 5=5EncodeWavToMp3,	6=6FinishedNoTags,	7=7ReAddTags,		8=8FinishedAll
	int	ret;				// return int (e.g. from qmessagebox)
	int iRunState;			// run state: 0=cannot create, 1=can create, 2=running, can stop(kill)
	int iPicsToDo;			// number of pictures to add to pdf
	int iProgsFound;		// 0/1=lame, 0x/1x=wavegain, 0xx/1xx=taglib
	int iTvCw0;				// tree view for file system width of column 0
	int iTvCw1;				// tree view for file system width of column 1
	int iTvCw2;				// tree view for file system width of column 2
	int iTvCw3;				// tree view for file system width of column 3
	int iWShown;			// popup widget currently shown: 0=none, 1=about-abouthelp, 2=help-abouthelp, (3=error-abouthelp, 4=info, 5=settings dialog)
	int iWToShow;			// popup widget to show: 0=none, 1=about-abouthelp, 2=help-abouthelp, (3=error-abouthelp, 4=info, 5= settings dialog)
	int iX;					// cursor position for messagebox
	int iY;					// cursor position for messagebox
      
	QAction *aboutAct;
	QAction *runAct;		// create normalized output
	QAction *disabledAct;
	QAction *exitAct;
	QAction *helpAct;
//	QAction *refreshAct;
	QAction *stopAct;

	QFileSystemModel *fsModel;

	QFile f;

	QFileInfo fi;

	QList<QModelIndex> lDirModSelIndexes; 		// list with indexes of selected pictures
	QList<QModelIndex> lDirModSelIndexesNew;	// list with new indexes of selected pictures

	FilePairs fpD;
	QList<FilePairs> liFilePairsMp3Wav;	// list struct with file name pairs successfully decoded from mp3 to wav
	QList<FilePairs> liFilePairsWavMp3;	// list struct with file name pairs successfully encoded from wav to to mp3 (normalized)
	QList<FilePairs> liFilePairsReTag;	// list struct with file name pairs worth trying to re-add tags
	QList<FilePairs> liFilePairsD;	// dummy list struct with file name pairs src n output for taglib to process

	QMenu *fileMenu;		// file menu
	QMenu *helpMenu;		// help menu
	QMenu *runMenu;			// run menu, create normalized output

	QModelIndex miIndex;	// index of last dir causing expand
	QModelIndex miD;		// dummy index in model/treeview
	QModelIndex miD2;		// dummy index in model/treeview

	QPixmap qPix;

	QPoint qPos;			// cursor position

	QString sD;				// dummy QString
	QString sD2;
	QString sDir;			// start dir for file dialogs

	QString sDirMp3Orig;	// full path and name of dir with mp3s to process
	QString sDirWav;		// full path and name of dir with wavs
	QString sDirMp3Norm;	// full path and name of dir with normalized mp3s

	QString sErr;
	QString sErr4;
	QString sErr8;
	QString sErr32;

	QString sFile;			// name of to be created output file
	QString sFileOld;		// needed to stop overwrite warning flood only check if file name changed
	QString sLastDir;		// last selected dir in treeview (from previous session)
	QString sOutDir;		// directory where to create the output, full path and incl. closing '/'
	QString sOutFullName;	// full path and name of output file
	QString sPath;			// full path of to be created pdf file
	QString sSelItem;		// full path and name or first selected item in tree view
	QString sSelItem0Name;	// name only of first selected item in tree view
	QString sSrcDir;		// full path and dir of last dir expanded in treeview
	QString sTmpDir;		// path to random temporary directory incl. / at end

	QStringList slD;
//	QStringList slDel;			// files to delete
	QStringList slMp3sToProc;	// list of mp3s to process, full path and name
	QStringList slWavsToProc;	// list of wavs to process, full path and name
	QStringList slErrors;		// list with files that could not processed correctly

	QToolBar *toolBar;

	aboutHelp *ahw;				// help widget
	lame *lm;					// lame qobject
	tags *tl;					// taglib qobject
	wavegain *wg;				// wavegain qobject


private slots:

//  **********  settings  **********
	void settingsRead();
	void settingsWrite();

//  **********  actions, menu, toolbar, statusbar  **********
	void actionsCreate();					// create actions
	void menuBarCreate();					// create menu
	void toolBarCreate();					// create toolbar
	void statusBarCreate();					// create statusbar
	void statusBarUpd(const QString& sD);	// update statusbar
	void updMnTlBr();						// update menu and toolbar

//  **********  message boxes  **********
	void messageBox(int iD, const QString sD, const QString sD2);	// show QMessageBox at cursor position, 1=Info, 2=Warning, 3=Critical
	void getCursorPos();					// get cursor position - iD=x , iD2=y

//  **********  about/help/error widget handling  **********
	void about();				// show about information
	void help();				// show help message
	void ahwCreate();			// create about/help/error dialog widget
	void ahwDel();				// hide=delete later about/help dialog widget

//  **********  quit  **********
	void quitDelay();			// delay to allow clean up - needed ????????????????????
	void quitPrep();			// quit clicked clean up
	void quitDo();				// finally quit the program

//  **********  init  **********
	void initDelay();			// short timer to allow GUI to come up before proceeding
	void init();				// starting
	void progStatClear();		// clear program status in widget
	void chkReqProgs();			// check for required programs (lame, wavegain, taglib)
	int chkDirsOnStart();		// check input and output dirs on start, whether they are still valid (writable)

//  **********  source file system model functions  **********
	void fsmCreate();									// create file system model
	void fsmExpandToLastDir();							// expand to dir used in last session
	void fsmSelectionChanged(const QModelIndex& miD);	// selection in tree view has changed

	void runStateUpd(int iD);	// update run state n menu n toolbar: iD: 0=lame missing, 1=wavegain missing, 2=tool for m4a missing
																	//	  5=no file in dir, 10=idle, 20=running
									//	  run states: 0=cannot create, 1=can create, 2=running, can stop(kill)

//  **********  normalization run functions  **********
	void runClicked();				// create normalized output clicked
	void runBegin();				// run begin, create dirs, get mp3s to process
	void runSuccess();				// run was successful - re-set Gui and states
	void runFailed();				// run failed - re-set Gui and states
	bool run1DirsCreate();			// create wav and normalized output directory
	bool run2GetMp3s();				// get list of mp3s to normalize
	void run3LameDecodeToWav();									// decode mp3s to wav with lame
	void run3LameDecodeToWavReturn(const QStringList& slD, const QList<FilePairs>& liFilePairsD);	// return from decode mp3s to wav with lame
	void run4WavegainNormalize();								// normalize wavs in dir
	void run4WavegainNormalizeReturn(bool bD);					// return from normalize wavs in dir
	void run5EncodeWavToMp3();									// encode wavs back to mp3s
	void run5EncodeWavToMp3Return(const QStringList& slD, const QList<FilePairs>& liFilePairsD);	// return from encode wavs back to mp3s
	void run7ReAddTags();										// re-add tags with taglib
	void run7ReAddTagsReturn(const QStringList& slD);	// return from re-add tags with taglib

	bool run1DelDirWav();						// try to delete wav directory if we created it
	bool run1DelDirOut();						// try to delete out directory if we created it
	bool run3DelFilesWav();						// try to delete created wav files in wav directory
	bool run5DelFilesMp3();						// try to delete created mp3 files in output directory
	void runGuiUpdFileProc(const QString& sD);	// update GUI - file currently processed
	void runGuiUpdRunLevel();					// update GUI according to current runlevel

	void destDirEditFin();						// manually setting the pdf path finished
	void destDirGetStartDir();					// get start dir for set dir dialog
	void destDirSet(const QString& sD);			// set dest dir in line widget, home dir if invalid
	void destDirSetClicked();					// button set destination dir clicked
	QString destDirStripDir(const QString& sD);	// get path from dir or file

	void outCreateSubDirToggled();	// create output sub dir checkbox toggled

	QString outputSetOutputFile();	// create full output file path & name
	bool outputChkOutputFile();		// check whether output file can be writen in destination dir and whether to overwrite file if file already exists
	void outputSetType();			// set output type, pdf or jpg
	void outputCreatePrep();		// prepare output creation

	void showErr();					// show any error during pdf creation

	void wavCreateSubDirToggled();	// create wav sub dir checkbox toggled

	void wCtrl(int iWToShow);	// show popup widget, if no other is shown: 0=none, 1=about-abouthelp, 2=help-abouthelp, (3=error-abouthelp, 4=info, 5=settings dialog)

	signals:
		void sigAhwShow(int iD);													// show info dlg: about, help
		void sigLameDo(const QString&, const QStringList&, const QString&, int, bool, bool, bool);
							// send src dir, file list with mp3s/wavs to process (name), output dir, command 1=decode/2=encode, randomize, del spaces, shorten to 8
		void sigWavegainStart(const QString&);										// send dir with wavs to normalize to process (full path and name)
		void sigTaglibReAdd(const QList<FilePairs>&);								// re-add tags, list with pairs of full path and name of original and normalized files
};

#endif 
