/***************************************************************************
 *   Copyright (C) 2009-2016 Helge Kortendieck  <alan@kasmithee.de>        *
 *                                                                         *
 *   taglib - get tags from original files and re-tag normalized files	   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "taglib.h"

tags::tags(QObject *parent) : QObject(parent)
{
	tagsClear();
}


tags::~tags()
{
}

// ******************************************************************
// re-adding tags - iterate over file lists src/dest
// ******************************************************************
void tags::tagsReAdd(const QList<FilePairs>& liTlFilePairs)	// re-add tags to normalized files, list struct with file name pairs src n output for taglib to process
{
	slErr.clear();
	for (iD = 0; iD < liTlFilePairs.count(); ++iD) {
		QFileInfo fiT(liTlFilePairs.at(iD).src);
			if (fiT.fileName().length() > 36) {					// limit GUI output to 30 character length
				emit sigTagsFileProc(fiT.fileName().left(18) + "..." + fiT.fileName().right(18));		// limit GUI update to 30 character length
			} else {
				emit sigTagsFileProc(fiT.fileName());
			}
cout << "tagsReAdd() : liFilePairs.at(iD) = " << QString::number(iD).toStdString() << "   liFilePairs.at(iD).src = " << liTlFilePairs.at(iD).src.toStdString() << endl;
		tagsGet(liTlFilePairs.at(iD).src);
cout << "tagsReAdd() : liFilePairs.at(iD) = " << QString::number(iD).toStdString() << "   liFilePairs.at(iD).out = " << liTlFilePairs.at(iD).out.toStdString() << endl;
		tagsWrite(liTlFilePairs.at(iD).out);
	}
	emit sigTagsReAddEnded(slErr);				// error handling in tagsWrite missing !!!
}

// ******************************************************************
// clear tags
// ******************************************************************
void tags::tagsClear()
{
	sArtist = "";
	sTitle = "";
	sAlbum = "";
	sYear = "";
	sComment = "";
	sTrack = "";
	sGenre = "";
	sComposer = "";
}

// ******************************************************************
// get tag info of song
// ******************************************************************
void tags::tagsGet(const QString& sD)
{
	sSongSrc = sD;
	tagsClear();
	if (!sSongSrc.isEmpty()) {				// don't crash if no song ever played
		TagLib::String str(TagLib::String(sSongSrc.toUtf8().data(), TagLib::String::UTF8));
		TagLib::FileRef f(str.toCString());
		if (f.tag()->isEmpty() == false) {		// is there any tag info?
			sArtist = TStringToQString(f.tag()->artist());
			sTitle = TStringToQString(f.tag()->title());
			sAlbum = TStringToQString(f.tag()->album());
			sYear = QString::number(f.tag()->year());
			sComment = TStringToQString(f.tag()->comment());
			sTrack = QString::number(f.tag()->track());
			sGenre = TStringToQString(f.tag()->genre());
		}
cout << "taglib tagsGet() : sArtist = " << sArtist.toStdString() << " , sTitle = " << sTitle.toStdString() << endl;

		TagLib::MPEG::File c(str.toCString());   // composer
		if (c.isValid()) {
			TagLib::ID3v2::Tag *id3v2tag = c.ID3v2Tag();
			if (id3v2tag) {
				TagLib::ID3v2::FrameList frameList = id3v2tag->frameListMap() ["TCOM"];
				if (!frameList.isEmpty()) {
					sComposer = frameList.front()->toString().toCString(true);
				}
			}
		}
	}
}

// ******************************************************************
// write tag info to song
// ******************************************************************
void tags::tagsWrite(const QString& sSongOut)	// save tags to mp3
{
	if (!sSongOut.isEmpty()) {				// don't crash if no song ever played
		TagLib::String str(TagLib::String(sSongOut.toUtf8().data(), TagLib::String::Latin1));
		TagLib::FileRef f(str.toCString());
		f.tag()->setArtist(TagLib::String(sArtist.toStdWString(), TagLib::String::UTF8));
		f.tag()->setTitle(TagLib::String(sTitle.toStdWString(), TagLib::String::UTF8));
		f.tag()->setAlbum(TagLib::String(sAlbum.toStdWString(), TagLib::String::UTF8));
		f.tag()->setYear(sYear.toInt());
		f.tag()->setComment(TagLib::String(sComment.toStdWString(), TagLib::String::UTF8));
		f.tag()->setTrack(sTrack.toInt());
		f.tag()->setGenre(TagLib::String(sGenre.toStdWString(), TagLib::String::UTF8));
		f.save();

		TagLib::MPEG::File c(str.toCString());	// composer
		if (c.isValid()) {
			TagLib::ID3v2::Tag *id3v2tag = c.ID3v2Tag();
			TagLib::ID3v2::TextIdentificationFrame *frame = new TagLib::ID3v2::TextIdentificationFrame("TCOM", TagLib::String::Latin1);
			frame->setText(TagLib::String(sComposer.toStdWString(), TagLib::String::UTF8));
			c.ID3v2Tag()->addFrame(frame);
			c.save();
		}
	} else {
		slErr.append(sSongSrc);
	}
}


#include "taglib.moc"
