/***************************************************************************
 *   Copyright (C) 2009-2016 Helge Kortendieck  <alan@kasmithee.de>		   *
 *                                                                         *
 *   wavegain - normalize wavs with wavegain, lossy						   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "wavegain.h"


wavegain::wavegain(QObject *parent) : QObject(parent)
{
//	bProc = false;
}

wavegain::~wavegain()
{
}

void wavegain::procPrep(const QString& sD)	// receive dir with files to process
{
	sWDir = sD;
	slArgs.clear();
	slArgs << "-r" << "-y" << "*.wav";
	procRun();
}

void wavegain::procRun()					// start wavegain qprocess to normalize all wavs in a directory
{
//  cout << "procRun: reached ! " << slArgs.at(0).toStdString() << endl;									// *****  debug  *****
	bOK = true;
	proc = new QProcess(this);
	proc->setWorkingDirectory(sWDir);
	connect(proc, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(procEnd(int, QProcess::ExitStatus)));
	connect(proc, SIGNAL(readyReadStandardOutput()), this, SLOT(procStdOut()));
	connect(proc, SIGNAL(readyReadStandardError()), this, SLOT(procStdErr()));
	proc->start("wavegain", slArgs);
}

void wavegain::procStdOut()				// read qprocess stdout
{
	aOutput = proc->readAllStandardOutput();		// read all stdard output into byte array
	if (!aOutput.isEmpty()) {
		sStdOut = QString(aOutput).trimmed();
		cout << "procStdOut: sStdOut: " << sStdOut.toStdString() << endl;											// *****  debug  *****
	}
}

void wavegain::procStdErr()				// read qprocess stderr
{
	aOutput = proc->readAllStandardError();		// read all errors into byte array
	if (!aOutput.isEmpty()) {
		sErr = QString(aOutput).trimmed();
		cout << "procStdErr: sErr: " << sErr.toStdString() << endl;												// *****  debug  *****
		if (sErr.length() > 36) {
			sErr = sErr.right(36);				// limit GUI output to 30 character length
		}
		emit sigNormFileProc(sErr);
		//bOK = false;								// we get output here which is not necessarily related to an error
	}
}

void wavegain::procEnd(int iD, QProcess::ExitStatus iD2)	// qprocess ended, start next if still program to check for
{
//	bProc = false;
//  cout << "procEnd: reached ! " << sTmpDir.toStdString() << endl;											// *****  debug  *****
	emit sigNormEnded(bOK);						// unable to detect any errors ?
}

void wavegain::procKill()					// kill qprocess, asked to stop
{
	if(proc) {
		proc->kill();
	}
}


#include "wavegain.moc"
